#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
  django_excel_export : Export your django database into Excel documents
  (C)2010 - Apidev - http://www.apidev.fr
  This is distributed under GNU LGPL license, see license.txt
"""

import models
from django.http import HttpResponse, Http404, HttpResponseForbidden
from django.db.models import get_model
from django.contrib.auth.decorators import user_passes_test
from django.conf import settings
import xlwt

def user_can_export(user):
    """export can only be done by admin users"""
    return user.is_staff

def dump_line(ws, line, fields):
    """format a line of the export"""
    style = xlwt.Style.default_style if line else xlwt.easyxf('font: bold 1; pattern: pattern solid, fore-colour gray25;')
    for col, f in enumerate(fields):
        if not (f is None):
            ws.write(line, col, unicode(f), style)

def guess_value_type(v):
    """Try to cast the value into the most obvious type"""
    try:
        if v[0]=="'" and v[-1]=="'":
            return v[1:-2]
        if v=='True' or v=='False':
            return bool(v)
        return int(v)
    except:
        return v

@user_passes_test(user_can_export, login_url='/admin/')
def export(request, name):
    """
    This view generates an excel file as defined in Export object
    It defines the database table, the fields to include and the filters to use
    The table-based export has limited capability but it is possible to make
    custom export by declaring an export function in the settings.py
    """
    
    #get the export info
    try:
        xp = models.Export.objects.get(name=name)
    except:
        raise Http404

    #create the excel document
    wb = xlwt.Workbook()
    ws = wb.add_sheet(name)

    if xp.table:
        #get the model of the table to be exported
        (app, table) = xp.table.split('.')
        the_model = get_model(app, table)
        
        #if some fields have been defined in the export: export only these columns
        fields = [k.col for k in xp.column_set.all()]
        if fields:
            labels = [k.label if k.label else k.col for k in xp.column_set.all()]
            dump_line(ws, 0, labels)
            
        #build the filter if defined
        if xp.filter_set.count()>0:
            qs = the_model.objects
            kwargs = {}
            for f in xp.filter_set.all():
                lookup = '{0}__{1}'.format(f.col, f.lookup)
                kwargs[lookup] = guess_value_type(f.value)
            qs = qs.filter(**kwargs)
        else:
            qs = the_model.objects.all()

        line = 0
        #for every line in the database, add a line in the excel file
        for m in qs:
            #if the fields are not defined: export all
            if not fields:
                fields = [f for f in m.__dict__.keys() if f!=u'_state']
                dump_line(ws, 0, fields)    
            line += 1
            values = []

            #write each field
            for k in fields:
                try:
                    v = m
                    for l in k.split('__'): #makes possible to access sub-fields ('a__b' --> a.b)
                        #foreign key: prints the value rather than the id
                        if v:
                            if l[-3:]=='_id':
                                v = getattr(v, l[:-3])
                            else:
                                v = getattr(v, l) #get the attribute of the object
                    #if ManyToMany: writes all in a single cell
                    if str(type(v)).find('django.db.models.fields.related.ManyRelatedManager')>=0:
                        values.append(', '.join([unicode(x) for x in v.all()]))
                    else:
                        values.append(v)
                except Exception, msg:
                    print msg
                    values.append('#ERROR#')

            dump_line(ws, line, values)
    elif xp.report:
        #Custom export: Execute a function (which must be declared in settings.py)
        if not xp.report in [x[0] for x in settings.EXCEL_REPORT_VIEWS]:
            raise Exception("report view must be in {0}".format([x[1] for x in settings.EXCEL_REPORT_VIEWS]))
        dot = xp.report.rfind('.')
        module, function = xp.report[:dot], xp.report[dot+1:]
        #execute the function
        exec("from {0} import {1}".format(module, function))
        data = eval(function)() #no arguments and must return a list of lists
        #write the returned data into the excel file
        for num, row in enumerate(data):
            dump_line(ws, num, row)
    
    #return the excel file as response
    response = HttpResponse(mimetype='application/vnd.ms-excel')
    response['Content-Disposition'] = 'attachment; filename={0}.xls'.format(name)
    wb.save(response)
    return response
    